==> iglu_python-0.2.1-py3-none-any.whl: iglu_python/grade_eugly.py <==
from typing import Union

import numpy as np
import pandas as pd

from .grade import _grade_formula
from .utils import check_data_columns


def grade_eugly(
    data: Union[pd.DataFrame, pd.Series, np.ndarray, list], lower: int = 70, upper: int = 140
) -> pd.DataFrame|float:
    """
    Calculate percentage of GRADE score attributable to target range.

    The function produces a DataFrame with values equal to the percentage of GRADE score
    attributed to values in the target range, i.e. values not below hypoglycemic or
    above hyperglycemic cutoffs.

    Parameters
    ----------
    data : Union[pd.DataFrame, pd.Series, np.ndarray, list]
        DataFrame with columns 'id', 'time', and 'gl', or a Series of glucose values,
        or a numpy array or list of glucose values
    lower : int, default=70
        Lower bound used for hypoglycemia cutoff, in mg/dL
    upper : int, default=140
        Upper bound used for hyperglycemia cutoff, in mg/dL

    Returns
    -------
    pd.DataFrame|float
        DataFrame with 1 row for each subject, a column for subject id and a column
        for GRADE euglycemia value. If a Series of glucose values is passed, then a float
        value is returned.

    References
    ----------
    Hill et al. (2007): A method for assessing quality of control
    from glucose profiles
    Diabetic Medicine 24: 753-758,
    doi:10.1111/j.1464-5491.2007.02119.x.

    Examples
    --------
    >>> data = pd.DataFrame({
    ...     'id': ['subject1', 'subject1', 'subject2', 'subject2'],
    ...     'time': ['2020-01-01 00:00:00', '2020-01-01 00:05:00',
    ...              '2020-01-01 00:00:00', '2020-01-01 00:05:00'],
    ...     'gl': [150, 200, 130, 190]
    ... })
    >>> data['time'] = pd.to_datetime(data['time'])
    >>> grade_eugly(data)
       id  GRADE_eugly
    0  subject1      45.67
    1  subject2      78.90

    >>> grade_eugly(data['gl'], lower=80, upper=180)
       GRADE_eugly
    0       65.43
    """
    # Handle Series input
    if isinstance(data, (pd.Series, np.ndarray, list)):
        if isinstance(data, (np.ndarray, list)):
            data = pd.Series(data)
        return grade_eugly_single(data, lower, upper)

    # Handle DataFrame input
    data = check_data_columns(data)

    # Calculate GRADE euglycemia for each subject
    out = data.groupby('id').agg(
        GRADE_eugly = ("gl", lambda x: grade_eugly_single(x, lower, upper))
    ).reset_index()

    return out


def grade_eugly_single(data: pd.Series, lower: int = 70, upper: int = 140) -> float:
    """Calculate GRADE euglycemia for a single timeseries"""
    data = data.dropna()
    if len(data) == 0:
        return np.nan

    # Calculate GRADE scores
    grade_scores = _grade_formula(data)

    # Calculate percentage in target range
    in_range = (data >= lower) & (data <= upper)
    total_grade = np.sum(grade_scores)
    if total_grade == 0:
        return np.nan

    eugly_percent = (np.sum(grade_scores[in_range]) / total_grade) * 100
    return eugly_percent

==> iglu_python-0.4.3-py3-none-any.whl: iglu_python/grade_eugly.py <==
from typing import Union

import numpy as np
import pandas as pd

from .grade import _grade_formula
from .utils import check_data_columns


def grade_eugly(
    data: Union[pd.DataFrame, pd.Series, np.ndarray, list], lower: int = 70, upper: int = 140
) -> pd.DataFrame | float:
    """
    Calculate percentage of GRADE score attributable to target range.

    The function produces a DataFrame with values equal to the percentage of GRADE score
    attributed to values in the target range, i.e. values not below hypoglycemic or
    above hyperglycemic cutoffs.

    Parameters
    ----------
    data : Union[pd.DataFrame, pd.Series, np.ndarray, list]
        DataFrame with columns 'id', 'time', and 'gl', or a Series of glucose values,
        or a numpy array or list of glucose values
    lower : int, default=70
        Lower bound used for hypoglycemia cutoff, in mg/dL
    upper : int, default=140
        Upper bound used for hyperglycemia cutoff, in mg/dL

    Returns
    -------
    pd.DataFrame|float
        DataFrame with 1 row for each subject, a column for subject id and a column
        for GRADE euglycemia value. If a Series of glucose values is passed, then a float
        value is returned.

    References
    ----------
    Hill et al. (2007): A method for assessing quality of control
    from glucose profiles
    Diabetic Medicine 24: 753-758,
    doi:10.1111/j.1464-5491.2007.02119.x.

    Examples
    --------
    >>> data = pd.DataFrame({
    ...     'id': ['subject1', 'subject1', 'subject2', 'subject2'],
    ...     'time': ['2020-01-01 00:00:00', '2020-01-01 00:05:00',
    ...              '2020-01-01 00:00:00', '2020-01-01 00:05:00'],
    ...     'gl': [150, 200, 130, 190]
    ... })
    >>> data['time'] = pd.to_datetime(data['time'])
    >>> grade_eugly(data)
       id  GRADE_eugly
    0  subject1      45.67
    1  subject2      78.90

    >>> grade_eugly(data['gl'], lower=80, upper=180)
       GRADE_eugly
    0       65.43
    """
    # Handle Series input
    if isinstance(data, (pd.Series, np.ndarray, list)):
        if isinstance(data, (np.ndarray, list)):
            data = pd.Series(data)
        return grade_eugly_single(data, lower, upper)

    # Handle DataFrame input
    data = check_data_columns(data)

    # Calculate GRADE euglycemia for each subject
    out = data.groupby("id").agg(GRADE_eugly=("gl", lambda x: grade_eugly_single(x, lower, upper))).reset_index()

    return out


def grade_eugly_single(data: pd.Series, lower: int = 70, upper: int = 140) -> float:
    """Calculate GRADE euglycemia for a single timeseries"""
    data = data.dropna()
    if len(data) == 0:
        return np.nan

    # Calculate GRADE scores
    grade_scores = _grade_formula(data)

    # Calculate percentage in target range
    in_range = (data >= lower) & (data <= upper)
    total_grade = np.sum(grade_scores)
    if total_grade == 0:
        return np.nan

    eugly_percent = (np.sum(grade_scores[in_range]) / total_grade) * 100
    return eugly_percent
